@@ -10,16 +10,18 @@
 
 #include <string.h>
 
-#include <omalloc/omalloc.h>
-#include <omalloc/omallocClass.h>
-#include <misc/mylimits.h>
+#include "omalloc/omalloc.h"
+#ifdef HAVE_OMALLOC
+#include "omalloc/omallocClass.h"
+#endif
 
+#include "misc/mylimits.h"
 
-#include <kernel/polys.h>
-#include <polys/operations/pShallowCopyDelete.h>
+#include "kernel/polys.h"
+#include "polys/operations/pShallowCopyDelete.h"
 
-#include <kernel/structs.h>
-#include <kernel/GBEngine/kstd1.h>   /* for s_poly_proc_t */
+#include "kernel/structs.h"
+#include "kernel/GBEngine/kstd1.h"   /* for s_poly_proc_t */
 
 // define if tailrings should be used
 #define HAVE_TAIL_RING
@@ -57,7 +59,7 @@ typedef struct denominator_list_s denominator_list_s;
 typedef denominator_list_s *denominator_list;
 
 struct denominator_list_s{number n; denominator_list next;};
-extern denominator_list DENOMINATOR_LIST;
+EXTERN_VAR denominator_list DENOMINATOR_LIST;
 
 class sTObject
 {
@@ -73,22 +75,27 @@ public:
     length,     // as of pLDeg
     pLength,    // either == 0, or == pLength(p)
     i_r;        // index of TObject in R set, or -1 if not in T
-  BOOLEAN is_normalized; // true, if pNorm was called on p, false otherwise
+
+#ifdef HAVE_SHIFTBBA
+  int shift;
+#endif
+
+  /*BOOLEAN*/ char is_normalized; // true, if pNorm was called on p, false otherwise
   // used in incremental sba() with F5C:
   // we know some of the redundant elements in
   // strat->T beforehand, so we can just discard
   // them and do not need to consider them in the
   // interreduction process
-  BOOLEAN is_redundant;
+  /*BOOLEAN*/ char is_redundant;
   // used in sba's sig-safe reduction:
   // sometimes we already know that a reducer
   // is sig-safe, so no need for a real
   // sig-safeness check
-  BOOLEAN is_sigsafe;
+  /*BOOLEAN*/ char is_sigsafe;
 
 
 #ifdef HAVE_PLURAL
-  BOOLEAN is_special; // true, it is a new special S-poly (e.g. for SCA)
+  /*BOOLEAN*/ char is_special; // true, it is a new special S-poly (e.g. for SCA)
 #endif
 
   // initialization
@@ -160,13 +167,14 @@ public:
   // manipulations
   KINLINE void pNorm();
   KINLINE void pCleardenom();
+  KINLINE void pContent();
 
 #ifdef KDEBUG
   void wrp();
 #endif
 };
 
-extern int strat_nr;
+EXTERN_VAR int strat_nr;
 
 class sLObject : public sTObject
 {
@@ -211,13 +219,13 @@ public:
   KINLINE void Tail_Minus_mm_Mult_qq(poly m, poly qq, int lq, poly spNoether);
   KINLINE void Tail_Mult_nn(number n);
   // deletes bucket, makes sure that p and t_p exists
-  KINLINE poly GetP(omBin lmBin = NULL);
+  KINLINE poly GetP(omBin lmBin = (omBin)NULL);
   // similar, except that only t_p exists
   KINLINE poly GetTP();
 
   // does not delete bucket, just canonicalizes it
   // returned poly is such that Lm(p) \in currRing, Tail(p) \in tailRing
-  KINLINE poly CanonicalizeP();
+  KINLINE void CanonicalizeP();
 
   // makes a copy of the poly of L
   KINLINE void Copy();
@@ -255,9 +263,12 @@ public:
 };
 
 
-extern int HCord;
+EXTERN_VAR int HCord;
 
-class skStrategy : public omallocClass
+class skStrategy
+#ifdef HAVE_OMALLOC
+                 : public omallocClass
+#endif
 {
 public:
   kStrategy next;
@@ -354,33 +365,33 @@ public:
   int blockredmax;
   #endif
   #ifdef HAVE_SHIFTBBA
-  int lV;
   int cv; // in shift bases: counting V criterion
+  /*BOOLEAN*/ char rightGB;
   #endif
-  BOOLEAN interpt;
-  BOOLEAN homog;
+  /*BOOLEAN*/ char interpt;
+  /*BOOLEAN*/ char homog;
 #ifdef HAVE_PLURAL
-  BOOLEAN z2homog; // Z_2 - homogeneous input allows product criterion in commutative and SCA cases!
+  /*BOOLEAN*/ char z2homog; // Z_2 - homogeneous input allows product criterion in commutative and SCA cases!
 #endif
-  BOOLEAN kHEdgeFound;
-  BOOLEAN honey,sugarCrit;
-  BOOLEAN Gebauer,noTailReduction;
-  BOOLEAN fromT;
-  BOOLEAN noetherSet;
-  BOOLEAN update;
-  BOOLEAN posInLOldFlag;
-  BOOLEAN use_buckets;
+  /*BOOLEAN*/ char kHEdgeFound;
+  /*BOOLEAN*/ char honey,sugarCrit;
+  /*BOOLEAN*/ char Gebauer,noTailReduction;
+  /*BOOLEAN*/ char fromT;
+  /*BOOLEAN*/ char noetherSet;
+  /*BOOLEAN*/ char update;
+  /*BOOLEAN*/ char posInLOldFlag;
+  /*BOOLEAN*/ char use_buckets;
   // if set, pLDeg(p, l) == (pFDeg(pLast(p), pLength)
-  BOOLEAN LDegLast;
+  /*BOOLEAN*/ char LDegLast;
   // if set, then L.length == L.pLength
-  BOOLEAN length_pLength;
+  /*BOOLEAN*/ char length_pLength;
   // if set, then posInL does not depend on L.length
-  BOOLEAN posInLDependsOnLength;
+  /*BOOLEAN*/ char posInLDependsOnLength;
   /*FALSE, if posInL == posInL10*/
 #ifdef HAVE_PLURAL
   // set this flag to 1 to stop the product criteria
   // use ALLOW_PROD_CRIT(strat) to test
-  BOOLEAN no_prod_crit;
+  /*BOOLEAN*/ char no_prod_crit;
 #define ALLOW_PROD_CRIT(A) (!(A)->no_prod_crit)
 #else
 #define ALLOW_PROD_CRIT(A) (1)
@@ -412,6 +423,7 @@ static inline LSet initL (int nr=setmaxL)
 void deleteInL(LSet set, int *length, int j,kStrategy strat);
 void enterL (LSet *set,int *length, int *LSetmax, LObject p,int at);
 void enterSBba (LObject &p,int atS,kStrategy strat, int atR = -1);
+void enterSBbaShift (LObject &p,int atS,kStrategy strat, int atR = -1);
 void enterSSba (LObject &p,int atS,kStrategy strat, int atR = -1);
 void initEcartPairBba (LObject* Lp,poly f,poly g,int ecartF,int ecartG);
 void initEcartPairMora (LObject* Lp,poly f,poly g,int ecartF,int ecartG);
@@ -476,27 +488,30 @@ int posInL10Ring (const LSet set, const int length,
              LObject* L,const kStrategy strat);
 int posInL110 (const LSet set, const int length,
              LObject* L,const kStrategy strat);
-KINLINE poly redtailBba (poly p,int pos,kStrategy strat,BOOLEAN normalize=FALSE);
-KINLINE poly redtailBbaBound (poly p,int pos,kStrategy strat,int bound,BOOLEAN normalize=FALSE);
+KINLINE poly redtailBba (poly p,int end_pos,kStrategy strat,BOOLEAN normalize=FALSE);
+KINLINE poly redtailBbaBound (poly p,int end_pos,kStrategy strat,int bound,BOOLEAN normalize=FALSE);
 #ifdef HAVE_RINGS
-KINLINE poly redtailBba_Z (poly p,int pos,kStrategy strat);
-poly redtailBba_Z (LObject* L, int pos, kStrategy strat );
+KINLINE poly redtailBba_Z (poly p,int end_pos,kStrategy strat);
+poly redtailBba_Z (LObject* L, int end_pos, kStrategy strat );
+void redtailBbaAlsoLC_Z (LObject* L, int end_pos, kStrategy strat );
 #endif
-poly redtailBba (LObject *L, int pos,kStrategy strat,
+poly redtailBba (LObject *L, int end_pos,kStrategy strat,
                  BOOLEAN withT = FALSE,BOOLEAN normalize=FALSE);
-poly redtailBbaBound (LObject *L, int pos,kStrategy strat,int bound,
+poly redtailBbaBound (LObject *L, int end_pos,kStrategy strat,int bound,
                  BOOLEAN withT = FALSE,BOOLEAN normalize=FALSE);
-poly redtailSba (LObject *L, int pos,kStrategy strat,
+poly redtailSba (LObject *L, int end_pos,kStrategy strat,
                  BOOLEAN withT = FALSE,BOOLEAN normalize=FALSE);
-poly redtailBba (TObject *T, int pos,kStrategy strat);
-poly redtail (poly p,int pos,kStrategy strat);
-poly redtail (LObject *L,int pos,kStrategy strat);
+poly redtailBba (TObject *T, int end_pos,kStrategy strat);
+poly redtail (poly p,int end_pos,kStrategy strat);
+poly redtail (LObject *L,int end_pos,kStrategy strat);
 poly redNF (poly h,int & max_ind,int nonorm,kStrategy strat);
 int redNF0 (LObject *P,kStrategy strat);
 poly redNFTail (poly h,const int sl,kStrategy strat);
 int redHoney (LObject* h, kStrategy strat);
+int redLiftstd (LObject* h, kStrategy strat);
 #ifdef HAVE_RINGS
 int redRing (LObject* h,kStrategy strat);
+int redRing_Z (LObject* h,kStrategy strat);
 int redRiloc (LObject* h,kStrategy strat);
 void enterExtendedSpoly(poly h,kStrategy strat);
 void enterExtendedSpolySig(poly h,poly hSig,kStrategy strat);
@@ -546,6 +561,7 @@ void initSyzRules (kStrategy strat);
 void updateS(BOOLEAN toT,kStrategy strat);
 void enterSyz (LObject &p,kStrategy strat, int atT);
 void enterT (LObject &p,kStrategy strat, int atT = -1);
+void replaceInLAndSAndT(LObject &p, int tj, kStrategy strat);
 #ifdef HAVE_RINGS
 void enterT_strong (LObject &p,kStrategy strat, int atT = -1);
 #endif
@@ -580,19 +596,24 @@ BOOLEAN findMinLMPair(poly sig, unsigned long not_sevSig, kStrategy strat, int s
 
 /// returns index of p in TSet, or -1 if not found
 int kFindInT(poly p, TSet T, int tlength);
+#ifdef HAVE_SHIFTBBA
+int kFindInTShift(poly p, TSet T, int tlength);
+#endif
 
 /// return -1 if no divisor is found
 ///        number of first divisor in T, otherwise
 int kFindDivisibleByInT(const kStrategy strat, const LObject* L, const int start=0);
+int kFindDivisibleByInT_Z(const kStrategy strat, const LObject* L, const int start=0);
+int kFindSameLMInT_Z(const kStrategy strat, const LObject* L, const int start=0);
 
+/// tests if T[0] divides the leading monomial of L, returns -1 if not
+int kTestDivisibleByT0_Z(const kStrategy strat, const LObject* L);
 /// return -1 if no divisor is found
 ///        number of first divisor in S, otherwise
 int kFindDivisibleByInS(const kStrategy strat, int *max_ind, LObject* L);
 
 int kFindNextDivisibleByInS(const kStrategy strat, int start,int max_ind, LObject* L);
-TObject*
-kFindDivisibleByInS(kStrategy strat, int pos, LObject* L, TObject *T,
-                    long ecart = LONG_MAX);
+TObject* kFindDivisibleByInS_T(kStrategy strat, int end_pos, LObject* L, TObject *T, long ecart = LONG_MAX);
 
 /***************************************************************
  *
@@ -629,7 +650,7 @@ BOOLEAN kTest(kStrategy strat);
 // test strat, and test that S is contained in T
 BOOLEAN kTest_TS(kStrategy strat);
 // test LObject
-BOOLEAN kTest_L(LObject* L, ring tailRing = NULL,
+BOOLEAN kTest_L(LObject* L, ring tailRing,
                  BOOLEAN testp = FALSE, int lpos = -1,
                  TSet T = NULL, int tlength = -1);
 // test TObject
@@ -641,7 +662,7 @@ BOOLEAN kTest_S(kStrategy strat);
 #define kTest_TS(A)     (TRUE)
 #define kTest_T(T)      (TRUE)
 #define kTest_S(T)      (TRUE)
-#define kTest_L(T)      (TRUE)
+#define kTest_L(T,R)    (TRUE)
 #endif
 
 
@@ -682,6 +703,29 @@ void f5c (kStrategy strat, int& olddeg, int& minimcnt, int& hilbeledeg,
 //         -1 tailRing change could not be performed due to exceeding exp
 //            bound of currRing
 int ksReducePoly(LObject* PR,
+                 TObject* PW,
+                 poly spNoether = NULL,
+                 number *coef = NULL,
+                 poly *mon =NULL,
+                 kStrategy strat = NULL);
+
+/* like ksReducePoly, but if the reducer has only 1 term we still
+ * compute a possible coefficient multiplier for PR. this comes from
+ * a special situation in redRing_Z and it is used only there. */
+int ksReducePolyZ(LObject* PR,
+                 TObject* PW,
+                 poly spNoether = NULL,
+                 number *coef = NULL,
+                 kStrategy strat = NULL);
+
+int ksReducePolyLC(LObject* PR,
+                 TObject* PW,
+                 poly spNoether = NULL,
+                 number *coef = NULL,
+                 kStrategy strat = NULL);
+
+
+int ksReducePolyGCD(LObject* PR,
                  TObject* PW,
                  poly spNoether = NULL,
                  number *coef = NULL,
@@ -801,10 +845,10 @@ ring sbaRing(kStrategy strat, const ring r=currRing, BOOLEAN complete=TRUE, int
 KINLINE void clearS (poly p, unsigned long p_sev, int* at, int* k,
   kStrategy strat);
 
-#include <kernel/GBEngine/kInline.h>
+#include "kernel/GBEngine/kInline.h"
 
 /* shiftgb stuff */
-#include <kernel/GBEngine/shiftgb.h>
+#include "kernel/GBEngine/shiftgb.h"
 
 poly pMove2CurrTail(poly p, kStrategy strat);
 
@@ -812,29 +856,37 @@ poly pMoveCurrTail2poly(poly p, kStrategy strat);
 
 poly pCopyL2p(LObject h, kStrategy strat);
 
-void enterTShift(LObject p, kStrategy strat, int atT, int uptodeg, int lV);
+void enterTShift(LObject p, kStrategy strat, int atT = -1);
 
-void initBuchMoraShift (ideal F,ideal Q,kStrategy strat);
+void enterOnePairShift (poly q, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int ecartq, int qisFromQ, int shiftcount, int ifromS);
 
-void enterOnePairSelfShifts (poly qq, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int uptodeg, int lV);
+void enterpairsShift (poly h,int k,int ecart,int pos,kStrategy strat, int atR);
 
-void enterOnePairShift (poly q, poly p, int ecart, int isFromQ, kStrategy strat, int atR, int ecartq, int qisFromQ, int shiftcount, int ifromS, int uptodeg, int lV); // ok
-
-void enterpairsShift (poly h,int k,int ecart,int pos,kStrategy strat, int atR,int uptodeg, int lV);
-
-void updateSShift(kStrategy strat,int uptodeg,int lV);
-
-void initBbaShift(kStrategy strat);
+void superenterpairsShift (poly h,int k,int ecart,int pos,kStrategy strat, int atR);
 
 poly redtailBbaShift (LObject* L, int pos, kStrategy strat, BOOLEAN withT, BOOLEAN normalize);
 
 int redFirstShift (LObject* h,kStrategy strat); // ok
 
-ideal freegb(ideal I, int uptodeg, int lVblock);
-
-ideal bbaShift(ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat, int uptodeg, int lV);
+ideal bbaShift(ideal F, ideal Q,intvec *w,intvec *hilb,kStrategy strat);
 // test syz strategy: // will be removed soon
-extern  int (*test_PosInT)(const TSet T,const int tl,LObject &h);
-extern  int (*test_PosInL)(const LSet set, const int length,
+EXTERN_VAR int (*test_PosInT)(const TSet T,const int tl,LObject &h);
+EXTERN_VAR int (*test_PosInL)(const LSet set, const int length,
                 LObject* L,const kStrategy strat);
+
+static inline void kDeleteLcm(LObject *P)
+{
+ if (P->lcm!=NULL)
+ {
+ #ifdef HAVE_RINGS
+   if (rField_is_Ring(currRing))
+     pLmDelete(P->lcm);
+   else
+ #endif
+     pLmFree(P->lcm);
+   P->lcm=NULL;
+ }
+}
+
+void initenterpairs (poly h,int k,int ecart,int isFromQ,kStrategy strat, int atR = -1);
 #endif
