@@ -996,6 +996,16 @@ static inline poly pp_Mult_mm(poly p, poly m, const ring r)
     return r->p_Procs->pp_Mult_mm(p, m, r);
 }
 
+// returns m*Copy(p), does neither destroy p nor m
+static inline poly pp_mm_Mult(poly p, poly m, const ring r)
+{
+  if (p==NULL) return NULL;
+  if (p_LmIsConstant(m, r))
+    return __pp_Mult_nn(p, pGetCoeff(m), r);
+  else
+    return r->p_Procs->pp_mm_Mult(p, m, r);
+}
+
 // returns p*m, destroys p, const: m
 static inline poly p_Mult_mm(poly p, poly m, const ring r)
 {
@@ -1006,6 +1016,16 @@ static inline poly p_Mult_mm(poly p, poly m, const ring r)
     return r->p_Procs->p_Mult_mm(p, m, r);
 }
 
+// returns m*p, destroys p, const: m
+static inline poly p_mm_Mult(poly p, poly m, const ring r)
+{
+  if (p==NULL) return NULL;
+  if (p_LmIsConstant(m, r))
+    return __p_Mult_nn(p, pGetCoeff(m), r);
+  else
+    return r->p_Procs->p_mm_Mult(p, m, r);
+}
+
 static inline poly p_Minus_mm_Mult_qq(poly p, const poly m, const poly q, int &lp, int lq,
                                       const poly spNoether, const ring r)
 {
@@ -1957,6 +1977,7 @@ static inline BOOLEAN p_IsConstant(const poly p, const ring r)
 /// either poly(1)  or gen(k)?!
 static inline BOOLEAN p_IsOne(const poly p, const ring R)
 {
+  if (p == NULL) return FALSE; /* TODO check if 0 == 1 */
   p_Test(p, R);
   return (p_IsConstant(p, R) && n_IsOne(p_GetCoeff(p, R), R->cf));
 }
